@@ -1,8 +1,9 @@
 /* GNOME-DB Components
- * Copyright (C) 2000-2002 The GNOME Foundation.
+ * Copyright (C) 2000 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,11 +20,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#if !defined(__dsn_config_h__)
-#  define __dsn_config_h__
+#ifndef __DSN_CONFIG_H__
+#define __DSN_CONFIG_H__
 
 #include <gtk/gtkwidget.h>
 
-GtkWidget *dsn_config_new (void);
+GtkWidget *dsn_config_new             (void);
+void       dsn_config_edit_properties (GtkWidget *dsn);
+void       dsn_config_delete          (GtkWidget *dsn);
 
 #endif
