@@ -22,6 +22,7 @@
 
 #include <string.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb-extra/libgnomedb-extra.h>
 #include <glade/glade-init.h>
 #include <glade/glade-build.h>
 
@@ -57,13 +58,15 @@ glade_module_register_widgets (void)
 	glade_register_widget (GNOME_DB_TYPE_CONNECTION_PROPERTIES,
 			       glade_standard_build_widget,
 			       NULL, NULL);
-	glade_register_widget (GNOME_DB_TYPE_DATA_SOURCE_SELECTOR,
+	glade_register_widget (GNOME_DB_TYPE_DSN_SELECTOR,
 			       glade_standard_build_widget,
 			       NULL, NULL);
-	glade_register_widget (GNOME_DB_TYPE_DSN_CONFIG_DRUID,
+#ifdef HAVE_GTKTWOTEN
+	glade_register_widget (GNOME_DB_TYPE_DSN_ASSISTANT,
 			       glade_standard_build_widget,
 			       NULL, NULL);
-	glade_register_widget (GNOME_DB_TYPE_DSN_CONFIG,
+#endif
+	glade_register_widget (GNOME_DB_TYPE_DSN_EDITOR,
 			       glade_standard_build_widget,
 			       NULL, NULL);
 	glade_register_widget (GNOME_DB_TYPE_EDITOR,
@@ -96,9 +99,6 @@ glade_module_register_widgets (void)
 	glade_register_widget (GNOME_DB_TYPE_PROVIDER_SELECTOR,
 			       glade_standard_build_widget,
 			       NULL, NULL);
-	glade_register_widget (GNOME_DB_TYPE_TABLE_EDITOR,
-			       glade_standard_build_widget,
-			       NULL, NULL);
 
 	glade_provide ("gnomedb");
 }
