@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -798,6 +798,42 @@ date_calendar_choose_cb (GtkWidget *button, GnomeDbEntryCommonTime *mgtim)
         position_popup (mgtim);
         gtk_grab_add (mgtim->priv->window);
         gtk_widget_show (mgtim->priv->window);
+
+	GdkScreen *screen;
+	gint swidth, sheight;
+	gint root_x, root_y;
+	gint wwidth, wheight;
+	gboolean do_move = FALSE;
+	screen = gtk_window_get_screen (GTK_WINDOW (mgtim->priv->window));
+	if (screen) {
+		swidth = gdk_screen_get_width (screen);
+		sheight = gdk_screen_get_height (screen);
+	}
+	else {
+		swidth = gdk_screen_width ();
+		sheight = gdk_screen_height ();
+	}
+	gtk_window_get_position (GTK_WINDOW (mgtim->priv->window), &root_x, &root_y);
+	gtk_window_get_size (GTK_WINDOW (mgtim->priv->window), &wwidth, &wheight);
+	if (root_x + wwidth > swidth) {
+		do_move = TRUE;
+		root_x = swidth - wwidth;
+	}
+	else if (root_x < 0) {
+		do_move = TRUE;
+		root_x = 0;
+	}
+	if (root_y + wheight > sheight) {
+		do_move = TRUE;
+		root_y = sheight - wheight;
+	}
+	else if (root_y < 0) {
+		do_move = TRUE;
+		root_y = 0;
+	}
+	if (do_move)
+		gtk_window_move (GTK_WINDOW (mgtim->priv->window), root_x, root_y);
+
         gtk_widget_grab_focus (mgtim->priv->date);
         popup_grab_on_window (mgtim->priv->window->window,
                               gtk_get_current_event_time ());
