@@ -467,7 +467,7 @@ gnome_db_combo_set_values_ext (GnomeDbCombo *combo, const GSList *values, gint *
 	g_return_val_if_fail (values, FALSE);
 
 	proxy = gnome_db_data_store_get_proxy (combo->priv->store);
-	row = gda_data_proxy_find_row_from_values (proxy, (GSList *) values, cols_index);
+	row = gda_data_model_get_row_from_values (GDA_DATA_MODEL (proxy), (GSList *) values, cols_index);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), row);
 
 	return (row >= 0) ? TRUE : FALSE;
