@@ -1,18 +1,18 @@
 /* gnome-db-raw-grid.c
  *
- * Copyright (C) 2002 - 2007 Vivien Malerba
+ * Copyright (C) 2002 - 2008 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -47,13 +47,13 @@ static void gnome_db_raw_grid_get_property (GObject *object,
 
 static void init_tree_view (GnomeDbRawGrid *grid);
 
-static void proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
 static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GnomeDbRawGrid *grid);
 static void proxy_row_updated_cb (GdaDataProxy *proxy, gint proxy_row, GnomeDbRawGrid *grid);
 static void proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid);
-static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *grid);
-static void paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbRawGrid *grid);
-static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
+static void paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid);
+static void paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+					     const gchar *att_name, const GValue *att_value, GnomeDbRawGrid *grid);
+static GError *iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawGrid *grid);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid);
 
 /* GnomeDbDataWidget interface */
@@ -71,16 +71,16 @@ static gboolean        gnome_db_raw_grid_widget_set_write_mode (GnomeDbDataWidge
 static GnomeDbDataWidgetWriteMode gnome_db_raw_grid_widget_get_write_mode (GnomeDbDataWidget *iface);
 
 typedef struct {
-	GdaParameterListGroup *group;
-	GtkCellRenderer       *data_cell;
-	GtkCellRenderer       *info_cell;
-	gboolean               info_shown;
-	gboolean               data_locked; /* TRUE if no modification allowed on that column */
+	GdaSetGroup     *group;
+	GtkCellRenderer *data_cell;
+	GtkCellRenderer *info_cell;
+	gboolean         info_shown;
+	gboolean         data_locked; /* TRUE if no modification allowed on that column */
 } ColumnData;
 
 #define COLUMN_DATA(x) ((ColumnData *)(x))
 
-static ColumnData *get_column_data (GnomeDbRawGrid *grid, GdaParameterListGroup *group);
+static ColumnData *get_column_data (GnomeDbRawGrid *grid, GdaSetGroup *group);
 
 struct _GnomeDbRawGridPriv
 {
@@ -382,8 +382,7 @@ gnome_db_raw_grid_set_property (GObject *object,
 				grid->priv->proxy = gnome_db_data_store_get_proxy (grid->priv->store);
 				grid->priv->data_model = gda_data_proxy_get_proxied_model (grid->priv->proxy);
 				
-				gda_object_connect_destroy (grid->priv->proxy,
-							    G_CALLBACK (proxy_destroyed_cb), grid);
+				g_object_ref (G_OBJECT (grid->priv->proxy));
 				g_signal_connect (grid->priv->proxy, "sample_changed",
 						  G_CALLBACK (proxy_sample_changed_cb), grid);
 				g_signal_connect (grid->priv->proxy, "row_updated",
@@ -395,13 +394,13 @@ gnome_db_raw_grid_set_property (GObject *object,
 				
 				g_signal_connect (grid->priv->iter, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), grid);
-				g_signal_connect (grid->priv->iter, "param_plugin_changed",
-						  G_CALLBACK (paramlist_param_plugin_changed_cb), grid);
+				g_signal_connect (grid->priv->iter, "holder-attr-changed",
+						  G_CALLBACK (paramlist_param_attr_changed_cb), grid);
 
 				g_signal_connect (grid->priv->iter, "row_changed",
 						  G_CALLBACK (iter_row_changed_cb), grid);
-				g_signal_connect (grid->priv->iter, "row_to_change",
-						  G_CALLBACK (iter_row_to_change_cb), grid);
+				g_signal_connect (grid->priv->iter, "validate-set",
+						  G_CALLBACK (iter_validate_set_cb), grid);
 
 				gda_data_model_iter_invalidate_contents (grid->priv->iter);
 							
@@ -555,23 +554,20 @@ init_tree_view (GnomeDbRawGrid *grid)
 	gint i;
 	GtkTreeView *tree_view;
 	GSList *list;
-	GdaDict *dict;
 
 	tree_view = GTK_TREE_VIEW (grid);
 
 	/* Creation of the columns in the treeview, to fit the parameters in grid->priv->iter param list */
-	dict = gda_object_get_dict (GDA_OBJECT (grid->priv->iter));
-	list = GDA_PARAMETER_LIST (grid->priv->iter)->groups_list;
+	list = GDA_SET (grid->priv->iter)->groups_list;
 	i = 0;
 	while (list) {
-		GdaParameter *param;
-		GdaParameterListGroup *group;
+		GdaHolder *param;
+		GdaSetGroup *group;
 		GtkTreeViewColumn *column;
 		GtkCellRenderer *renderer;
 		ColumnData *column_data;
-		gboolean explicit_hide = FALSE;
 
-		group = GDA_PARAMETER_LIST_GROUP (list->data);
+		group = GDA_SET_GROUP (list->data);
 
 		/* update the list of columns data */
 		column_data = get_column_data (grid, group);
@@ -592,17 +588,18 @@ init_tree_view (GnomeDbRawGrid *grid)
 			
 			nodes = group->nodes;
 			while (nodes && nullok) {
-				if (gda_parameter_get_not_null (GDA_PARAMETER 
-								(GDA_PARAMETER_LIST_NODE (nodes->data)->param)))
+				if (gda_holder_get_not_null (GDA_HOLDER (GDA_SET_NODE (nodes->data)->holder)))
 					nullok = FALSE;
 				nodes = g_slist_next (nodes);
 			}
 			
 			/* determine title */
 			if (g_slist_length (group->nodes) == 1)
-				title = (gchar *) gda_object_get_name (GDA_OBJECT (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param));
+				title = (gchar *) g_object_get_data (G_OBJECT (GDA_SET_NODE (group->nodes->data)->holder),
+								     "name");
 			else 
-				title = (gchar *) gda_object_get_name (GDA_OBJECT (group->nodes_source->data_model));
+				title = (gchar *) g_object_get_data (G_OBJECT (group->nodes_source->data_model),
+								     "name");
 
 			if (title)
 				title = replace_double_underscores (g_strdup (title));
@@ -612,7 +609,7 @@ init_tree_view (GnomeDbRawGrid *grid)
 
 			/* FIXME: if nullok is FALSE, then set the column title in bold */
 
-			renderer = gnome_db_data_cell_renderer_combo_new (GDA_PARAMETER_LIST (grid->priv->iter), group->nodes_source);
+			renderer = gnome_db_data_cell_renderer_combo_new (GDA_SET (grid->priv->iter), group->nodes_source);
 			column_data->data_cell = renderer;
 			gtk_tree_view_insert_column_with_data_func (tree_view, i, title, renderer,
 								    (GtkTreeCellDataFunc) cell_renderer_value_set_attributes, 
@@ -624,50 +621,45 @@ init_tree_view (GnomeDbRawGrid *grid)
 					  G_CALLBACK (data_cell_values_changed), grid);
 
 			g_object_set_data (G_OBJECT (column), "source", group->nodes_source);
-			/* FIXME: compute @explicit_hide */
 		}
 		else {
 			/* single direct parameter */
 			GType g_type;
-			gchar *plugin = NULL;
+			const gchar *plugin = NULL;
+			const GValue *plugin_val;
 			gchar *title;
 			
-			param = GDA_PARAMETER (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
-			g_type = gda_parameter_get_g_type (param);
+			param = GDA_HOLDER (GDA_SET_NODE (group->nodes->data)->holder);
+			g_type = gda_holder_get_g_type (param);
 
-			title = (gchar *) gda_object_get_name (GDA_OBJECT (param));
+			g_object_get (G_OBJECT (param), "name", &title, NULL);
 			if (title && *title)
-				title = replace_double_underscores (g_strdup (title));
+				title = replace_double_underscores (title);
 			else
 				title = NULL;
 			if (!title)
 				title = g_strdup (_("No title"));
 			
-			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
-			renderer = gnome_db_util_new_cell_renderer (dict, g_type, plugin);
+			plugin_val = gda_holder_get_attribute (param, GNOME_DB_ATTRIBUTE_PLUGIN);
+			if (plugin_val) {
+				if (G_VALUE_TYPE (plugin_val) == G_TYPE_STRING)
+					plugin = g_value_get_string (plugin_val);
+				else
+					g_warning (_("The '%s' attribute should be a G_TYPE_STRING value"),
+						   GNOME_DB_ATTRIBUTE_PLUGIN);
+			}
+			renderer = gnome_db_util_new_cell_renderer (g_type, plugin);
 			column_data->data_cell = renderer;
-			if (! column_data->data_locked)
-				column_data->data_locked = GDA_PARAMETER_LIST_NODE (group->nodes->data)->hint & 
-					GDA_PARAMETER_LIST_PARAM_READ_ONLY;
 			gtk_tree_view_insert_column_with_data_func (tree_view, i, title, renderer,
 								    (GtkTreeCellDataFunc) cell_renderer_value_set_attributes, 
 								    grid, NULL);
 			column = gtk_tree_view_get_column (tree_view, i);
 			g_free (title);
 
-			if (GDA_PARAMETER_LIST_NODE (group->nodes->data)->hint & GDA_PARAMETER_LIST_PARAM_HIDE)
-				g_object_set_data (G_OBJECT (param), "_gnome_db_explicit_show", GINT_TO_POINTER (1));
-
-			if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "_gnome_db_explicit_show")) == 1)
-				explicit_hide = TRUE;
-
 			g_signal_connect (G_OBJECT (renderer), "changed", 
 					  G_CALLBACK (data_cell_value_changed), grid);
 		}
 
-		if (explicit_hide)
-			gtk_tree_view_column_set_visible (column, FALSE);
-
 		g_object_set_data (G_OBJECT (column), "data_renderer", renderer);
 		g_object_set (G_OBJECT (renderer), "editable", !column_data->data_locked, NULL);
 
@@ -698,29 +690,6 @@ init_tree_view (GnomeDbRawGrid *grid)
 		list = g_slist_next (list);
 		i++;
 	}
-
-	/*
-	 * Hiding some columns of the grid
-	 */
-	list = GDA_PARAMETER_LIST (grid->priv->iter)->nodes_list;
-	while (list) {
-		guint hint = GDA_PARAMETER_LIST_NODE (list->data)->hint;
-
-		if (hint & GDA_PARAMETER_LIST_PARAM_HIDE) {
-			GtkTreeViewColumn *viewcol;
-			GdaParameterListGroup *group;
-			gint pos;
-
-			group = gda_parameter_list_find_group_for_param ((GdaParameterList *) grid->priv->iter, 
-									 GDA_PARAMETER_LIST_NODE (list->data)->param);
-			pos = g_slist_index (((GdaParameterList *) grid->priv->iter)->groups_list, group);
-			g_assert (pos >= 0);
-
-			viewcol = gtk_tree_view_get_column (GTK_TREE_VIEW (grid), pos);
-			gtk_tree_view_column_set_visible (viewcol, FALSE);
-		}
-		list = g_slist_next (list);
-	}
 }
 
 /*
@@ -733,7 +702,7 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 				    GtkTreeModel *tree_model,
 				    GtkTreeIter *iter, GnomeDbRawGrid *grid)
 {
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	guint attributes;
 	gboolean to_be_deleted = FALSE;
 	ColumnData *column_data;
@@ -744,7 +713,7 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
 		GList *values = NULL;
-		GdaParameterListSource *source;
+		GdaSetSource *source;
 
 		source = g_object_get_data (G_OBJECT (tree_column), "source");
 
@@ -805,8 +774,8 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 		offset = gda_data_model_get_n_columns (gda_data_proxy_get_proxied_model (grid->priv->proxy));
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->store), iter, 
 				    DATA_STORE_COL_TO_DELETE, &to_be_deleted,
 				    DATA_STORE_COL_MODEL_ROW, &row,
@@ -834,7 +803,7 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 				   GtkTreeModel *tree_model,
 				   GtkTreeIter *iter, GnomeDbRawGrid *grid)
 {
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	guint attributes;
 	gboolean to_be_deleted = FALSE;
 	ColumnData *column_data;
@@ -844,7 +813,7 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 	
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
-		GdaParameterListSource *source;
+		GdaSetSource *source;
 
 		source = g_object_get_data (G_OBJECT (tree_column), "source");
 
@@ -868,8 +837,8 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 		offset = gda_data_model_get_n_columns (gda_data_proxy_get_proxied_model (grid->priv->proxy));
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->store), iter, 
 				    DATA_STORE_COL_TO_DELETE, &to_be_deleted,
 				    DATA_STORE_COL_MODEL_ROW, &row,
@@ -896,22 +865,17 @@ static void
 data_cell_value_changed (GtkCellRenderer *renderer, const gchar *path, const GValue *new_value, GnomeDbRawGrid *grid)
 {
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	
 	group = g_object_get_data (G_OBJECT (renderer), "group");
 	g_assert (g_slist_length (group->nodes) == 1);
 
 	if (set_iter_from_path (grid, path, &iter)) {
 		    gint col;
-		    col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								    GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		    col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					 GDA_SET_NODE (group->nodes->data)->holder);
 		    gnome_db_data_store_set_value (grid->priv->store, &iter, col, new_value);
 	}
-
-#ifdef debug_no
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 5);
-#endif
 }
 
 
@@ -925,7 +889,7 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 			  GSList *new_values, GSList *all_new_values, GnomeDbRawGrid *grid)
 {
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	
 	group = g_object_get_data (G_OBJECT (renderer), "group");
 	g_assert (group->nodes_source);
@@ -936,11 +900,6 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		/* the reason for not having any value is that the GnomeDbDataCellRendererCombo had no selected item */
 		return;
 	
-#ifdef debug_NO
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
-
 	if (set_iter_from_path (grid, path, &iter)) {
 		GSList *list, *params;
 		gint col, proxy_row;
@@ -948,14 +907,12 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		proxy_row = gnome_db_data_store_get_row_from_iter (grid->priv->store, &iter);
 
 		/* update the GnomeDbDataStore */
-		params = group->nodes;
-		list = new_values;
-		while (list) {
-			col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (params->data)->param);
+		for (params = group->nodes, list = new_values; 
+		     list; 
+		     params = params->next, list = list->next) {
+			col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					     GDA_SET_NODE (params->data)->holder);
 			gnome_db_data_store_set_value (grid->priv->store, &iter, col, (GValue *)(list->data));
-			list = g_slist_next (list);
-			params = g_slist_next (params);
 		}
 		
 #ifdef PROXY_STORE_EXTRA_VALUES
@@ -972,11 +929,6 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 		}
 #endif
 	}
-
-#ifdef debug_NO
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
 }
 
 static gboolean
@@ -1000,39 +952,23 @@ set_iter_from_path (GnomeDbRawGrid *grid, const gchar *path, GtkTreeIter *iter)
 static void
 treeview_column_clicked_cb (GtkTreeViewColumn *tree_column, GnomeDbRawGrid *grid)
 {
-	/* 
-	const gchar *filter;
-	gchar *newfilter = NULL;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
+	GSList *nodes;
 
 	group = g_object_get_data (G_OBJECT (tree_column), "group");
 	g_assert (group);
-	filter = gda_data_proxy_get_filter_expr (grid->priv->proxy);
-	if (filter) {
-		TO_IMPLEMENT;
-	}
-	else {
-		GString *string;
-		GSList *nodes;
 
-		string = g_string_new ("ORDER BY ");
-		for (nodes = group->nodes; nodes; nodes = nodes->next) {
-			GdaParameter *param = ((GdaParameterListNode*) nodes->data)->param;
-			g_assert (param);
-			if (nodes != group->nodes)
-				g_string_append (string, ", ");
-			g_string_append (string, gda_object_get_name (GDA_OBJECT (param)));
+	for (nodes = group->nodes; nodes; nodes = nodes->next) {
+		GdaHolder *param = ((GdaSetNode*) nodes->data)->holder;
+		gint pos;
+		g_assert (param);
+		
+		pos = g_slist_index (GDA_SET (grid->priv->iter)->holders, param);
+		if (pos >= 0) {
+			gda_data_proxy_set_ordering_column (grid->priv->proxy, pos, NULL);
+			break;
 		}
-		newfilter = string->str;
-		g_string_free (string, FALSE);
 	}
-	
-	if (newfilter) {
-		g_print ("FILTER: %s\n", newfilter);
-		g_free (newfilter);
-	}
-	*/
-	TO_IMPLEMENT;
 }
 
 /*
@@ -1044,7 +980,7 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 {
 	GtkTreePath *treepath;
 	GtkTreeIter iter;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 	GtkTreeModel *tree_model;
 	gint col;
 	gint offset;
@@ -1067,14 +1003,13 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 	if (group->nodes_source) {
 		/* parameters depending on a GdaDataModel */
 		gint proxy_row;
-		GSList *list = group->nodes;
+		GSList *list;
 		proxy_row = gnome_db_data_store_get_row_from_iter (grid->priv->store, &iter);
 
-		while (list) {
-			col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (list->data)->param);
+		for (list = group->nodes; list; list = list->next) {
+			col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+					     GDA_SET_NODE (list->data)->holder);
 			gnome_db_data_store_set_value (grid->priv->store, &iter, offset + col, attribute);
-			list = g_slist_next (list);
 		}
 
 #ifdef PROXY_STORE_EXTRA_VALUES
@@ -1108,16 +1043,11 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GdaValue
 		gint col;
 
 		g_assert (g_slist_length (group->nodes) == 1);
-		col = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-								GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+		col = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+				     GDA_SET_NODE (group->nodes->data)->holder);
 		gnome_db_data_store_set_value (grid->priv->store, &iter, offset + col, attribute);
 	}
 	gda_value_free (attribute);
-
-#ifdef debug_no
-	DEBUG_HEADER;
-	gda_object_dump (grid->priv->proxy, 0);
-#endif
 }
 
 static void
@@ -1203,7 +1133,7 @@ action_commit_cb (GtkAction *action, GnomeDbRawGrid *grid)
 					      actually succedded the commit 
 					      => we need to come back to that row
 					   */
-				gda_data_model_iter_set_at_row (grid->priv->iter, row);
+				gda_data_model_iter_move_at_row (grid->priv->iter, row);
 		}
 	}
 	else
@@ -1408,7 +1338,7 @@ action_filter_cb (GtkAction *action, GnomeDbRawGrid *grid)
 	/* move the filter window to a correct location */
 	/* FIXME: let the user specify the position function like GtkTreeView -> search_position_func() */
 	gtk_grab_add (grid->priv->filter_window);
-	filter_position_func (GTK_TREE_VIEW (grid), grid->priv->filter_window, NULL);
+	filter_position_func (GTK_WIDGET (grid), grid->priv->filter_window, NULL);
 	gtk_widget_show (grid->priv->filter_window);
 	popup_grab_on_window (grid->priv->filter_window->window,
 			      gtk_get_current_event_time ());	
@@ -1705,8 +1635,8 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		GList *columns, *list;
 		gint *cols, nb_cols;
 		gint *rows = NULL, nb_rows = 0;
-		GdaParameter *param;
-		GdaParameterList *paramlist;
+		GdaHolder *param;
+		GdaSet *paramlist;
 
 		types = g_object_get_data (G_OBJECT (dialog), "types");
 		filename = g_object_get_data (G_OBJECT (dialog), "filename");
@@ -1720,17 +1650,13 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		list = columns;
 		while (list) {
 			if (gtk_tree_view_column_get_visible (GTK_TREE_VIEW_COLUMN (list->data))) {
-				GdaParameterListGroup *group;
+				GdaSetGroup *group;
 				GSList *params;
 
 				group = g_object_get_data (G_OBJECT (list->data), "group");
-				params = group->nodes;
-				while (params) {
-					cols [nb_cols] = gda_data_model_iter_get_column_for_param (grid->priv->iter, 
-									GDA_PARAMETER_LIST_NODE (params->data)->param);
-					nb_cols ++;
-					params = g_slist_next (params);
-				}
+				for (params = group->nodes; params; nb_cols ++, params = params->next) 
+					cols [nb_cols] = g_slist_index (((GdaSet *)grid->priv->iter)->holders,
+									GDA_SET_NODE (params->data)->holder);
 			}
 			list = g_list_next (list);
 		}
@@ -1761,9 +1687,9 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 		grid->priv->export_type = export_type;
 		switch (export_type) {
 		case 0:
-			param = gda_parameter_new_string ("SEPARATOR", "\t");
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+			param = gda_holder_new_string ("SEPARATOR", "\t");
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
@@ -1771,9 +1697,9 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			g_object_unref (paramlist);
 			break;
 		case 1:
-			param = gda_parameter_new_string ("SEPARATOR", ",");
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+			param = gda_holder_new_string ("SEPARATOR", ",");
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_TEXT_SEPARATED, 
@@ -1782,11 +1708,11 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 			break;
 		case 2:
 			param = NULL;
-			body = (gchar *) gda_object_get_name (GDA_OBJECT (grid->priv->data_model));
+			body = (gchar *) g_object_get_data (G_OBJECT (grid->priv->data_model), "name");
 			if (body)
-				param = gda_parameter_new_string ("NAME", body);
-			paramlist = gda_parameter_list_new (NULL);
-			gda_parameter_list_add_param (paramlist, param);
+				param = gda_holder_new_string ("NAME", body);
+			paramlist = gda_set_new (NULL);
+			gda_set_add_holder (paramlist, param);
 			g_object_unref (param);
 			body = gda_data_model_export_to_string (GDA_DATA_MODEL (grid->priv->data_model), 
 								GDA_DATA_MODEL_IO_DATA_ARRAY_XML, 
@@ -1813,7 +1739,7 @@ save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbRawGrid *grid)
 					}
 				}
 
-				if (! gda_file_save (path, body, strlen (body))) {
+				if (! g_file_set_contents (path, body, strlen (body), NULL)) {
 					gnome_db_show_error (NULL, _("Could not save file %s"), path);
 					g_free (body);
 					g_free (path);
@@ -1929,7 +1855,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 		has_selection = gtk_tree_selection_get_selected (selection, &model, &iter) ? 1 : 0;
 
 	if (has_selection == 1) {
-		if (!gda_data_model_iter_set_at_row (grid->priv->iter, 
+		if (!gda_data_model_iter_move_at_row (grid->priv->iter, 
 						     gnome_db_data_store_get_row_from_iter (grid->priv->store, 
 											    &iter))) {
 			/* selection changing is refused, return to the current selected row */
@@ -1944,7 +1870,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 	else {
 		/* render all the parameters invalid, and make the iter point to row -1 */
 		gda_data_model_iter_invalidate_contents (grid->priv->iter);
-		gda_data_model_iter_set_at_row (grid->priv->iter, -1);
+		gda_data_model_iter_move_at_row (grid->priv->iter, -1);
 	}
 
 #ifdef debug_signal
@@ -1961,7 +1887,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbRawGrid *gri
 }
 
 static ColumnData *
-get_column_data (GnomeDbRawGrid *grid, GdaParameterListGroup *group)
+get_column_data (GnomeDbRawGrid *grid, GdaSetGroup *group)
 {
 	ColumnData *retval = NULL;
 	GSList *list = grid->priv->columns_data;
@@ -2025,9 +1951,9 @@ static void
 gnome_db_raw_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gboolean editable)
 {
 	GnomeDbRawGrid *grid;
-	GdaParameter *param;
+	GdaHolder *param;
 	ColumnData *column_data;
-	GdaParameterListGroup *group;
+	GdaSetGroup *group;
 
 	g_return_if_fail (iface && GNOME_DB_IS_RAW_GRID (iface));
 	grid = GNOME_DB_RAW_GRID (iface);
@@ -2036,10 +1962,10 @@ gnome_db_raw_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gb
 	if (grid->priv->data_model) {
 		editable = editable && !gda_data_proxy_is_read_only (grid->priv->proxy);
 
-		param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+		param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 		g_return_if_fail (param);
 
-		group = gda_parameter_list_find_group_for_param (GDA_PARAMETER_LIST (grid->priv->iter), param);
+		group = gda_set_get_group (GDA_SET (grid->priv->iter), param);
 		g_return_if_fail (group);
 
 		column_data = get_column_data (grid, group);
@@ -2062,15 +1988,15 @@ gnome_db_raw_grid_show_column_actions (GnomeDbDataWidget *iface, gint column, gb
 	g_return_if_fail (grid->priv);
 
 	if (column >= 0) {
-		GdaParameter *param;
-		GdaParameterListGroup *group;
+		GdaHolder *param;
+		GdaSetGroup *group;
 		ColumnData *cdata;
 
 		/* setting applies only to the @column column */
-		param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+		param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 		g_return_if_fail (param);
 		
-		group = gda_parameter_list_find_group_for_param (GDA_PARAMETER_LIST (grid->priv->iter), param);
+		group = gda_set_get_group (GDA_SET (grid->priv->iter), param);
 		g_return_if_fail (group);
 
 		cdata = get_column_data (grid, group);
@@ -2101,25 +2027,24 @@ gnome_db_raw_grid_col_set_show (GnomeDbDataWidget *iface, gint column, gboolean
 	GnomeDbRawGrid *grid;
 	gint pos = -1;
 	GtkTreeViewColumn *viewcol;
-	GdaParameterListGroup *group;
-	GdaParameter *param;
+	GdaSetGroup *group;
+	GdaHolder *param;
 
 	g_return_if_fail (iface && GNOME_DB_IS_RAW_GRID (iface));
 	grid = GNOME_DB_RAW_GRID (iface);
 	g_return_if_fail (grid->priv);
 
-	param = gda_data_model_iter_get_param_for_column (grid->priv->iter, column);
+	param = gda_data_model_iter_get_holder_for_field (grid->priv->iter, column);
 	g_return_if_fail (param);
 
-	group = gda_parameter_list_find_group_for_param ((GdaParameterList *)grid->priv->iter, param);	
-	pos = g_slist_index (((GdaParameterList *)grid->priv->iter)->groups_list, group);	
+	group = gda_set_get_group ((GdaSet *)grid->priv->iter, param);	
+	pos = g_slist_index (((GdaSet *)grid->priv->iter)->groups_list, group);	
 	g_assert (pos >= 0);
 
 	viewcol = gtk_tree_view_get_column (GTK_TREE_VIEW (grid), pos);
 
 	/* Sets the column's visibility */
 	gtk_tree_view_column_set_visible (viewcol, shown);
-	g_object_set_data (G_OBJECT (param), "_gnome_db_explicit_show", GINT_TO_POINTER (shown ? 2 : 1));
 }
 
 
@@ -2162,7 +2087,7 @@ gnome_db_raw_grid_widget_get_gda_model (GnomeDbDataWidget *iface)
 }
 
 static void
-paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *grid)
+paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbRawGrid *grid)
 {
 	GList *columns, *list;
 
@@ -2192,33 +2117,62 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbRawGrid *g
 }
 
 static void
-paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbRawGrid *grid)
+paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, 
+				 const gchar *att_name, const GValue *att_value, GnomeDbRawGrid *grid)
 {
-	/* TODO: be more specific and change only the cell renderer corresponding to @param */
-	paramlist_public_data_changed_cb (paramlist, grid);
+	if (!strcmp (att_name, GNOME_DB_ATTRIBUTE_PLUGIN)) {
+		/* TODO: be more specific and change only the cell renderer corresponding to @param */
+
+		/* keep a list of hidden columns */
+		gint i;
+		gboolean *hidden_cols;
+		GList *cols, *list;
+		cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
+		hidden_cols = g_new0 (gboolean, g_list_length (cols));
+		for (list = cols, i = 0; list; list = list->next, i++) {
+			if (!gtk_tree_view_column_get_visible (GTK_TREE_VIEW_COLUMN (list->data)))
+				hidden_cols [i] = TRUE;
+		}
+		g_list_free (cols);
+
+		/* re-create columns */
+		paramlist_public_data_changed_cb (paramlist, grid);
+
+		/* hide columns which were hidden */
+		cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid));
+		for (list = cols, i = 0; list; list = list->next, i++) {
+			if (hidden_cols [i])
+				gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN (list->data), FALSE);
+		}
+		g_list_free (cols);
+		g_free (hidden_cols);
+	}
 }
 
-static gboolean
-iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawGrid *grid)
+static GError *
+iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawGrid *grid)
 {
-	gboolean allow_change = TRUE;
-	if (row >= 0) {
-		if (grid->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
-			/* write back the current row */
-			if (gda_data_proxy_row_has_changed (grid->priv->proxy, row)) {
-				GError *error = NULL;
-				if (!gda_data_proxy_apply_row_changes (grid->priv->proxy, row, &error)) {
-					allow_change = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) grid, 
-													  error);
-					if (allow_change)
-						gda_data_proxy_cancel_row_changes (grid->priv->proxy, row, -1);
-					g_error_free (error);
-				}
+GError *error = NULL;
+	gint row = gda_data_model_iter_get_row (iter);
+	
+	if (row < 0)
+		return NULL;
+
+	if ((grid->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) &&
+		/* write back the current row */
+	    gda_data_proxy_row_has_changed (grid->priv->proxy, row) &&
+	    !gda_data_proxy_apply_row_changes (grid->priv->proxy, row, &error)) {
+		if (gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) grid, 
+										error)) {
+			gda_data_proxy_cancel_row_changes (grid->priv->proxy, row, -1);
+			if (error) {
+				g_error_free (error);
+				error = NULL;
 			}
 		}
 	}
 	
-	return allow_change;
+	return error;	
 }
 
 static void
@@ -2298,17 +2252,6 @@ proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid)
 	g_object_unref (G_OBJECT (proxy));
 }
 
-static void
-proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbRawGrid *grid)
-{
-	g_assert (proxy == grid->priv->proxy);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_destroyed_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_sample_changed_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_row_updated_cb), grid);
-	g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (proxy_reset_cb), grid);
-	grid->priv->proxy = NULL;
-}
-
 static void
 gnome_db_raw_grid_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model)
 {
@@ -2352,18 +2295,23 @@ gnome_db_raw_grid_clean (GnomeDbRawGrid *grid)
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
 						      G_CALLBACK (paramlist_public_data_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
-						      G_CALLBACK (paramlist_param_plugin_changed_cb), grid);
+						      G_CALLBACK (paramlist_param_attr_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
 						      G_CALLBACK (iter_row_changed_cb), grid);
 		g_signal_handlers_disconnect_by_func (grid->priv->iter,
-						      G_CALLBACK (iter_row_to_change_cb), grid);
+						      G_CALLBACK (iter_validate_set_cb), grid);
 		g_object_unref (grid->priv->iter);
 		grid->priv->iter = NULL;
 	}
 	
 	/* proxy */
-	if (grid->priv->proxy)
-		proxy_destroyed_cb (grid->priv->proxy, grid);
+	if (grid->priv->proxy) {
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_sample_changed_cb), grid);
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_row_updated_cb), grid);
+		g_signal_handlers_disconnect_by_func (grid->priv->proxy, G_CALLBACK (proxy_reset_cb), grid);
+		g_object_unref (grid->priv->proxy);
+		grid->priv->proxy = NULL;
+	}
 
 	/* store */
 	if (grid->priv->store) {
