@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2006 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -211,6 +211,12 @@ static gboolean popup_menu_cb (GtkWidget *button, GnomeDbEntryPict *mgpict);
 static gboolean event_cb (GtkWidget *button, GdkEvent *event, GnomeDbEntryPict *mgpict);
 static void size_allocate_cb (GtkWidget *wid, GtkAllocation *allocation, GnomeDbEntryPict *mgpict);
 
+static void
+realize_cb (GnomeDbEntryPict *mgpict, GnomeDbEntryWrapper *mgwrap)
+{
+	display_image (mgpict, NULL, NULL, NULL);
+}
+
 static GtkWidget *
 create_entry (GnomeDbEntryWrapper *mgwrap)
 {
@@ -258,6 +264,9 @@ create_entry (GnomeDbEntryWrapper *mgwrap)
 
 	display_image (mgpict, NULL, GTK_STOCK_MISSING_IMAGE, _("No data to display"));
 
+	g_signal_connect (G_OBJECT (mgpict), "realize",
+			  G_CALLBACK (realize_cb), mgwrap);
+
 	return vbox;
 }
 
@@ -353,8 +362,7 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 	}
 
 	/* fill in mgpict->priv->data */
-	if (!common_pict_load_data (&(mgpict->priv->options), value, &(mgpict->priv->bindata), &error)) {
-		stock = GINT_TO_POINTER (error->code);
+	if (!common_pict_load_data (&(mgpict->priv->options), value, &(mgpict->priv->bindata), &stock, &error)) {
 		notice_msg = g_strdup (error->message ? error->message : "");
 		g_error_free (error);
 	}
@@ -367,7 +375,7 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 static void 
 display_image (GnomeDbEntryPict *mgpict, const GValue *value, const gchar *error_stock, const gchar *notice)
 {
-	gchar *stock = (gchar *) error_stock;
+	const gchar *stock = error_stock;
 	gchar *notice_msg = NULL;
 	GdkPixbuf *pixbuf;
 	PictAllocation alloc;
@@ -380,7 +388,8 @@ display_image (GnomeDbEntryPict *mgpict, const GValue *value, const gchar *error
 	if (pixbuf)
 		g_object_ref (pixbuf);
 	else {
-		pixbuf = common_pict_make_pixbuf (&(mgpict->priv->options), &(mgpict->priv->bindata), &alloc, &error);
+		pixbuf = common_pict_make_pixbuf (&(mgpict->priv->options), &(mgpict->priv->bindata), &alloc, 
+						  &stock, &error);
 		if (pixbuf) 
 			common_pict_add_cached_pixbuf (&(mgpict->priv->options), value, pixbuf);
 	}
@@ -391,7 +400,6 @@ display_image (GnomeDbEntryPict *mgpict, const GValue *value, const gchar *error
 	}
 	else {
 		if (error) {
-			stock = GINT_TO_POINTER (error->code);
 			notice_msg = g_strdup (error->message ? error->message : "");
 			g_error_free (error);
 		}
@@ -407,11 +415,11 @@ display_image (GnomeDbEntryPict *mgpict, const GValue *value, const gchar *error
 	if (notice || notice_msg) {
 		gtk_label_set_text (GTK_LABEL (mgpict->priv->notice), notice ? notice : notice_msg);
 		gtk_widget_show (mgpict->priv->notice);
+		g_free (notice_msg);
 	}
-	else  
+	else 
 		gtk_widget_hide (mgpict->priv->notice);
 
-	g_free (notice_msg);
 	common_pict_adjust_menu_sensitiveness (&(mgpict->priv->popup_menu), mgpict->priv->editable, &(mgpict->priv->bindata));
 	gtk_widget_queue_resize ((GtkWidget *) mgpict);
 }
