@@ -930,13 +930,17 @@ build_form_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_n
 		GdaSet *plist;
 		GdaHolder *param;
 		GnomeDbDataWidget *datawid;
+		GValue *value;
 
 		wid = gnome_db_form_new (model);
 		g_object_get (G_OBJECT (wid), "raw-form", &datawid, NULL);
 		g_object_set (G_OBJECT (datawid), "show-actions", TRUE, NULL);
 		plist = GDA_SET (gnome_db_data_widget_get_current_data (GNOME_DB_DATA_WIDGET (datawid)));
 		param = plist->holders->data;
-		g_object_set (G_OBJECT (param), "entry-plugin", plugin_name, NULL);
+
+		value = gda_value_new_from_string (plugin_name, G_TYPE_STRING);
+		gda_holder_set_attribute (param, GNOME_DB_ATTRIBUTE_PLUGIN, value);
+		gda_value_free (value);
 		g_object_unref (datawid);
 	}
 	else {
@@ -962,13 +966,17 @@ build_grid_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_n
 		GdaSet *plist;
 		GdaHolder *param;
 		GnomeDbDataWidget *datawid;
-
+		GValue *value;
+		
 		wid = gnome_db_grid_new (model);
 		g_object_get (G_OBJECT (wid), "raw-grid", &datawid, NULL);
 		g_object_set (G_OBJECT (datawid), "info-cell-visible", TRUE, NULL);
 		plist = GDA_SET (gnome_db_data_widget_get_current_data (GNOME_DB_DATA_WIDGET (datawid)));
 		param = plist->holders->data;
-		g_object_set (G_OBJECT (param), "entry-plugin", plugin_name, NULL);
+
+		value = gda_value_new_from_string (plugin_name, G_TYPE_STRING);
+		gda_holder_set_attribute (param, GNOME_DB_ATTRIBUTE_PLUGIN, value);
+		gda_value_free (value);
 		g_object_unref (datawid);
 	}
 	else {
